import random
from character import humanoid, table
     
# -- retrieve the cost of the cheapest rank
def minimum_cost (attributes, race):
    minimum = 1000
    
    for attribute in attributes [:]:
        cost = table.advancement.cost (race, attribute)
        if minimum > cost: minimum = cost

    return minimum


# -- randomly level up a creature
def level_up (char, points):
    attributes = table.advancement.attributes (char.Race)
    number = len (attributes)
    
    # -- determine the minimum cost for leveling up attributes
    min_cost = minimum_cost (attributes, char.Race)
    
    while points >= min_cost:
        attribute = attributes[random.randrange (0, number)]
        
        # -- do not raise attributes that are too costly
        if points < table.advancement.cost (char.Race, attribute): 
            continue
        
        # -- do not exceed maximum
        if char.increase (attribute, 1):
            points = points - table.advancement.cost (char.Race, attribute)
        else:
            # -- attribute can't be raised any more
            attributes.remove (attribute)

            # -- are there attributes to raise left?
            number = len (attributes)
            if number == 0: return
            
            # -- recalculate minimum
            min_cost = minimum_cost (attributes, char.Race)

        
# -- test code
wolf = humanoid.humanoid ("Wolf")

while 1:
    level = input ("Select level of creature (0 to quit): ")
    if level <= 0: break 
    
    # -- add a random modifier
    attributes = table.advancement.attributes (wolf.Race)
    attribute = attributes[random.randrange (0, len (attributes))]
    print "\n Added +1 to %s\n" % attribute
    
    wolf.add_modifier (attribute, 1)
    level_up (wolf, level * 5)
    
    # -- set derived attributes to maximum
    wolf.Health = wolf.MaxHealth
    wolf.Power = wolf.MaxPower
    
    print " Strength/Max = %i / %i" % (wolf.Strength, wolf.MaxStrength)
    print " Hardness/Max = %i / %i" % (wolf.Hardness, wolf.MaxHardness)
    print "  Agility/Max = %i / %i" % (wolf.Agility, wolf.MaxAgility)
    print "Willpower/Max = %i / %i" % (wolf.Willpower, wolf.MaxWillpower)
    print "   Health/Max = %i / %i" % (wolf.Health, wolf.MaxHealth)
    print "    Power/Max = %i / %i" % (wolf.Power, wolf.MaxPower)
    print "---------------------"

    # -- cleanup
    wolf.remove_modifier (attribute, 1)
    wolf.Strength = table.advancement.initial (wolf.Race, "Strength")
    wolf.Hardness = table.advancement.initial (wolf.Race, "Hardness")
    wolf.Agility = table.advancement.initial (wolf.Race, "Agility")
    wolf.Willpower = table.advancement.initial (wolf.Race, "Willpower")
    
