#
#  $Id$
#
#  (C) Copyright 2003 Kai Sterker <kaisterker@linuxgames.com>
#  Part of the Adonthell Project http://adonthell.linuxgames.com
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY.
#
#  See the COPYING file for more details
#

import adonthell

#
# This class implements a global storage for character related
# constants. Note that it is static and should not be instanciated.
# (For that reason, it requires Python 2.2 or later to work)
#
class advancement (object):
    # -- dictionary storing the advancement data with the 
    #    race as key. The advancement data itself is a dictionary
    #    with attribute as key and a tuple containing cost to raise
    #    the attribute and maximum rank allowed for the attribute.
    __Table = {}
    
    
    # -- speaking names for attribute-tuple access
    COST    = 0
    MAXIMUM = 1
    INITIAL = 2
    
    
    # -- Return the attributes available to a given race
    def attributes (race):
        return advancement.__Table.get (race, {}).keys ()
        
        
    # -- return the maximum rank the given race may gain 
    #    in the given attribute
    def maximum (race, attribute):
        return advancement.lookup (race, attribute, advancement.MAXIMUM, 0)
        
        
    # -- return the cost to advance the given attribute 
    #    of the given race
    def cost (race, attribute):
        return advancement.lookup (race, attribute, advancement.COST, 1000)


    # -- return initial value of the given attribute
    def initial (race, attribute):
        return advancement.lookup (race, attribute, advancement.INITIAL, 0)
        
    
    # -- actual lookup operation
    def lookup (race, attribute, key, errval):
        try:
            dict = advancement.__Table[race]
        except:
            print "*** warning: character.table.advancement: race '%s' not in table" % str (race) 
            return errval
            
        try:
            return dict[attribute][key]
        except:
            print "*** warning: character.table.advancement: attribute '%s' not in table of race '%s'" % (attribute, race) 
            return errval    
    
    
    # -- load advancement table from disk
    def get_state ():
        file = adonthell.igzstream ()
        file.open ("advancement.data")
        
        if not file.is_open ():
            print "*** error: character.table.advancement: cannot open 'advancement.data'"
            return
        
        more_races = file.get_bool ()
        
        while more_races:
            race = file.get_string ()
            more_attributes = file.get_bool ()
            attributes = {}
            
            while more_attributes:
                attribute = file.get_string ()
                cost = file.get_uint16 ()
                maximum = file.get_uint16 ()
                initial = file.get_uint16 ()
                
                attributes[attribute] = (cost, maximum, initial)
                    
                more_attributes = file.get_bool ()

            advancement.__Table[race] = attributes
            more_races = file.get_bool ()
    
        file.close ()
    
    # -- make all methods static
    attributes = staticmethod (attributes)
    get_state = staticmethod (get_state)
    maximum = staticmethod (maximum)
    initial = staticmethod (initial)
    lookup = staticmethod (lookup)
    cost = staticmethod (cost)
    

# Load the advancement data from disk. Happens when this module
# is imported for the first time
advancement.get_state ()


#
# This class contains formulas to calculate the maximum of
# certain attributes from other attributes at runtime.
#
class variable (object):
    __Table = {\
        "Melee Attack":         lambda (char): 10 * char.Strength,\
        "Melee Damage":         lambda (char): 1 * char.Strength, \
        "Carrying Capacity":    lambda (char): 10 * char.Strength, \
        "Health":               lambda (char): 10 * char.Hardness, \
        "Physical Resistance":  lambda (char): 1 * char.Hardness, \
        "Poison Resistance":    lambda (char): 1 * char.Hardness, \
        "Fire Resistance":      lambda (char): 1 * char.Hardness, \
        "Ranged Attack":        lambda (char): 10 * char.Agility, \
        "Dodge":                lambda (char): 10 * char.Agility, \
        "Parry":                lambda (char): 10 * char.Agility, \
        "Power":                lambda (char): 10 * char.Willpower, \
        "Mental resistance":    lambda (char): 1 * char.Willpower, \
    }
    
    def maximum (character, attribute):
        return variable.__Table[attribute](character)
        
    maximum = staticmethod (maximum)
